--***************************************************************
--* File         : TB_IPLSPI.vhd                                *
--* Created      : time-stamp: <08/03/03 02:28:50 kawasan>      *
--* Modified     : Time-stamp: <08/04/02 01:26:32 kawasan>      *
--*								*
--* Test-Bench                                                  *
--*								*
--* Copyright 2008 Osamu Kawashima. all rights reserved		*
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use std.textio.all;
use ieee.std_logic_textio.all;

entity TB_IPLSPI is end;

architecture behavior of TB_IPLSPI is
    
	component IPLSPI
	generic (
		SADD_SIZE       : integer := 32;
		DADD_SIZE       : integer := 32
	);
	port (
		RESET           : in  std_logic;
		SYSCLK          : in  std_logic;

		S_SADD          : in  std_logic_vector(SADD_SIZE-1 downto 0);
		D_SADD          : in  std_logic_vector(DADD_SIZE-1 downto 0);
		D_EADD          : in  std_logic_vector(DADD_SIZE-1 downto 0);
		GO              : in  std_logic;
		DONE            : out std_logic;

		SPI_S_N         : out std_logic;	-- active=Low
		SPI_CLK         : out std_logic;
		SPI_DO          : out std_logic;
		SPI_DI          : in  std_logic;

		D_ADD           : out std_logic_vector(DADD_SIZE-1 downto 0);
		D_DO            : out std_logic_vector(7 downto 0);
		D_WE            : out std_logic
	);
	end component;

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	constant SADD_SIZE      : integer := 32;
	constant DADD_SIZE      : integer := 32;

	constant WT1            : Time := 10 ns;
	constant WT_DELAY       : Time := 1 ns;
	constant WT_IDLE        : Time := 100 ns;
	constant CLK_CYCLE      : Time := 21 ns;	-- about 48MHz

	signal RESET            : std_logic := '1';
	signal SYSCLK           : std_logic := '0';
	signal S_SADD           : std_logic_vector(SADD_SIZE-1 downto 0) := (others=>'0');
	signal D_SADD           : std_logic_vector(DADD_SIZE-1 downto 0) := (others=>'0');
	signal D_EADD           : std_logic_vector(DADD_SIZE-1 downto 0) := (others=>'0');
	signal GO               : std_logic := '0';
	signal DONE             : std_logic := '0';
	signal SPI_S_N          : std_logic := '0';
	signal SPI_CLK          : std_logic := '0';
	signal SPI_DO           : std_logic := '0';
	signal SPI_DI           : std_logic := 'Z';
	signal D_ADD            : std_logic_vector(DADD_SIZE-1 downto 0) := (others=>'0');
	signal D_DO             : std_logic_vector(7 downto 0) := (others=>'0');
	signal D_WE             : std_logic := '0';

begin
    
	uIPLSPI : IPLSPI
	generic map (
		SADD_SIZE       => SADD_SIZE,
		DADD_SIZE       => DADD_SIZE
	)
	port map (
		RESET           => RESET,
		SYSCLK          => SYSCLK,

		S_SADD          => S_SADD,
		D_SADD          => D_SADD,
		D_EADD          => D_EADD,
		GO              => GO,
		DONE            => DONE,

		SPI_S_N         => SPI_S_N,
		SPI_CLK         => SPI_CLK,
		SPI_DO          => SPI_DO,
		SPI_DI          => SPI_DI,

		D_ADD           => D_ADD,
		D_DO            => D_DO,
		D_WE            => D_WE
	);

	process begin
		SYSCLK <= '0';
		wait for CLK_CYCLE/2;
		SYSCLK <= '1';
		wait for CLK_CYCLE/2;
	end process;

	process
		file VEC        : text is in "spi.vec";
		variable L      : line;
		variable D_IN   : std_logic_vector(7 downto 0);
	begin
		readline (VEC, L);
		read (L, D_IN);

		for iCnt in 7 downto 0 loop
			wait until SPI_CLK'event and SPI_CLK='0';
			wait for WT_DELAY;
			if SPI_S_N='0' then
				SPI_DI <= D_IN(iCnt);
			end if;

			wait until SPI_CLK'event and SPI_CLK='1';
			wait for WT_DELAY;
		end loop;

		if endfile(VEC) then
			SPI_DI <= 'Z';
			wait;
		end if;
	end process;

	process
		variable ln         : line;
	begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;
		RESET <= '0';

		-- Reset
		wait for WT_IDLE;
		S_SADD <= x"00000000";
		D_SADD <= x"00000100";
		D_EADD <= x"00000109";
		GO <= '1';
		wait for CLK_CYCLE;
		GO <= '0';

		-- End
		write(ln, string'("--- End of Test-Bench. ---"));
		writeline(output, ln);
		wait;
	end process;

end behavior;
